#include "MCP79410.h"
#include "i2c.h"

void rtcReadRegister(uint8_t reg, uint8_t * data) {
	i2cStart();
	i2cSendAddress(MCP79410_RTC_WRITE_ADDRESS);
	i2cSendData(reg);
	i2cStart();
	i2cSendAddress(MCP79410_RTC_READ_ADDRESS);
	i2cReceive(1, data);
	i2cStop();
}

void rtcWriteRegister(uint8_t reg, uint8_t data) {
	i2cStart();
	i2cSendAddress(MCP79410_RTC_WRITE_ADDRESS);
	i2cSendData(reg);
	i2cSendData(data);
	i2cStop();
}

void rtcReadRegisters(uint8_t reg, uint8_t * data, uint8_t size) {
	i2cStart();
	i2cSendAddress(MCP79410_RTC_WRITE_ADDRESS);
	i2cSendData(reg);
	i2cStart();
	i2cSendAddress(MCP79410_RTC_READ_ADDRESS);
	for (uint8_t i = 0; i < size; i++) {
		i2cReceive(i == size - 1, data++);
	}
	i2cStop();
}

void rtcWriteRegisters(uint8_t reg, uint8_t * data, uint8_t size) {
	i2cStart();
	i2cSendAddress(MCP79410_RTC_WRITE_ADDRESS);
	i2cSendData(reg);
	for (uint8_t i = 0; i < size; i++) {
		i2cSendData(data[i]);
	}
	i2cStop();
}

void eepromReadByte(uint8_t reg, uint8_t * data) {
	i2cStart();
	i2cSendAddress(MCP79410_EEPROM_WRITE_ADDRESS);
	i2cSendData(reg);
	i2cStart();
	i2cSendAddress(MCP79410_EEPROM_READ_ADDRESS);
	i2cReceive(1, data);
	i2cStop();
}

void eepromWriteByte(uint8_t reg, uint8_t data) {
	i2cStart();
	i2cSendAddress(MCP79410_EEPROM_WRITE_ADDRESS);
	i2cSendData(reg);
	i2cSendData(data);
	i2cStop();
}

void eepromReadBytes(uint8_t reg, uint8_t * data, uint8_t size) {
	i2cStart();
	i2cSendAddress(MCP79410_EEPROM_WRITE_ADDRESS);
	i2cSendData(reg);
	i2cStart();
	i2cSendAddress(MCP79410_EEPROM_READ_ADDRESS);
	for (uint8_t i = 0; i < size; i++) {
		i2cReceive(i == size - 1, data++);
	}
	i2cStop();
}

void eepromWriteBytes(uint8_t reg, uint8_t * data, uint8_t size) {
	i2cStart();
	i2cSendAddress(MCP79410_EEPROM_WRITE_ADDRESS);
	i2cSendData(reg);
	for (uint8_t i = 0; i < size; i++) {
		i2cSendData(data[i]);
	}
	i2cStop();
}

uint8_t eepromIsBusy(void) {
	i2cStart();
	uint8_t status = i2cSendAddress(MCP79410_EEPROM_WRITE_ADDRESS);
	i2cStop();
	return status != I2C_OK;
}

void eepromWriteProtected(uint8_t reg, uint8_t * data, uint8_t size) {
	rtcWriteRegister(MCP79410_EEUNLOCK, 0x55);
	rtcWriteRegister(MCP79410_EEUNLOCK, 0xAA);
	eepromWriteBytes(reg, data, size);
}

uint8_t fromBCD(uint8_t val) {
	return ((val & 0b11110000) >> 4) * 10 + (val & 0b00001111);
}

uint8_t toBCD(uint8_t val) {
	return ((val / 10) << 4) | (val % 10);
}

void rtcReadDateTime(DateTime * dateTime) {
	RtcDateTimeRegisters rtcDateTimeRegisters;
	rtcReadRegisters(MCP79410_RTCSEC, (uint8_t *) &rtcDateTimeRegisters, sizeof(rtcDateTimeRegisters));
	dateTime->seconds = fromBCD(rtcDateTimeRegisters.RTCSEC & 0b01111111);
	dateTime->minutes = fromBCD(rtcDateTimeRegisters.RTCMIN & 0b01111111);
	dateTime->hours = fromBCD(rtcDateTimeRegisters.RTCHOUR & 0b01111111);
	dateTime->dayOfWeek = fromBCD(rtcDateTimeRegisters.RTCWKDAY & 0b00000111);
	dateTime->dayOfMonth = fromBCD(rtcDateTimeRegisters.RTCDATE & 0b00111111);
	dateTime->month = fromBCD(rtcDateTimeRegisters.RTCMTH & 0b00011111);
	dateTime->year = 2000 + fromBCD(rtcDateTimeRegisters.RTCYEAR & 0b11111111);
}

void rtcWriteDateTime(DateTime * dateTime) {
	rtcWriteRegister(MCP79410_RTCSEC, 0);
	RtcDateTimeRegisters rtcDateTimeRegisters;
	rtcDateTimeRegisters.RTCSEC = toBCD(dateTime->seconds);
	rtcDateTimeRegisters.RTCMIN = toBCD(dateTime->minutes);
	rtcDateTimeRegisters.RTCHOUR = toBCD(dateTime->hours);
	rtcDateTimeRegisters.RTCWKDAY = toBCD(dateTime->dayOfWeek) | _BV(MCP79410_VBATEN);
	rtcDateTimeRegisters.RTCDATE = toBCD(dateTime->dayOfMonth);
	rtcDateTimeRegisters.RTCMTH = toBCD(dateTime->month);
	rtcDateTimeRegisters.RTCYEAR = toBCD(dateTime->year - 2000);
	rtcWriteRegisters(MCP79410_RTCSEC, (uint8_t *) &rtcDateTimeRegisters, sizeof(rtcDateTimeRegisters));
	rtcWriteRegister(MCP79410_RTCSEC, rtcDateTimeRegisters.RTCSEC | _BV(MCP79410_ST));
}

void rtcReadPowerDownDateTime(DateTime * dateTime) {
	RtcPowerRegisters rtcPowerRegisters;
	rtcReadRegisters(MCP79410_PWRDNMIN, (uint8_t *) &rtcPowerRegisters, sizeof(rtcPowerRegisters));
	dateTime->minutes = fromBCD(rtcPowerRegisters.PWRMIN & 0b01111111);
	dateTime->hours = fromBCD(rtcPowerRegisters.PWRHOUR & 0b00111111);
	dateTime->dayOfMonth = fromBCD(rtcPowerRegisters.PWRDATE & 0b00111111);
	dateTime->month = fromBCD(rtcPowerRegisters.PWRMTH & 0b00011111);
}

void rtcReadPowerUpDateTime(DateTime * dateTime) {
	RtcPowerRegisters rtcPowerRegisters;
	rtcReadRegisters(MCP79410_PWRUPMIN, (uint8_t *) &rtcPowerRegisters, sizeof(rtcPowerRegisters));
	dateTime->minutes = fromBCD(rtcPowerRegisters.PWRMIN & 0b01111111);
	dateTime->hours = fromBCD(rtcPowerRegisters.PWRHOUR & 0b00111111);
	dateTime->dayOfMonth = fromBCD(rtcPowerRegisters.PWRDATE & 0b00111111);
	dateTime->month = fromBCD(rtcPowerRegisters.PWRMTH & 0b00011111);
}